/**
 * 
 */
package it.unicam.cs.tesei.graphs;

import java.util.Iterator;
import java.util.Set;

/**
 * Interfaccia per un generico grafo i cui nodi sono etichettati con elementi
 * della classe E e i cui archi sono etichettati con elementi della classe V.
 * 
 * @author Luca Tesei
 *
 */
public interface Graph<E, V> {

    /**
     * Colore bianco di un nodo del grafo.
     */
    public static int COLOR_WHITE = 0;

    /**
     * Colore grigio di un nodo del grafo.
     */
    public static int COLOR_GREY = 0;

    /**
     * Colore nero di un nodo del grafo.
     */
    public static int COLOR_BLACK = 0;

    /**
     * Ritorna il numero di nodi di questo grafo.
     * 
     * @return in numero di nodi di questo grafo.
     */
    public int size();

    /**
     * Determina se questo grafo è vuoto.
     * 
     * @return se questo grafo è vuoto, false altrimenti.
     */
    public boolean isEmpty();

    /**
     * Cancella tutti i nodi e gli archi di questo grafo portandolo ad essere un
     * grafo vuoto.
     */
    public void clear();

    /**
     * Determina se c'è un nodo in questo grafo etichettato con un oggetto dato.
     * 
     * @param label
     *            l'oggetto della classe E che viene cercato tra i nodi del
     *            grafo.
     * @return true se esiste un nodo di questo grafo etichettato con l'oggetto
     *         label.
     * @throws NullPointerException
     *             se l'oggetto passato è null.
     */
    public boolean containsNode(E label);

    /**
     * Aggiunge un nodo a questo grafo, etichettato con un oggetto dato.
     * 
     * @param label
     *            l'oggetto che etichetta il nuovo nodo.
     * @return true se l'oggetto passato non è presente nel grafo, false
     *         altrimenti. In quest'ultimo caso il nodo non viene aggiunto.
     * @throws NullPointerException
     *             se l'oggetto passato è null.
     */
    public boolean addNode(E label);

    /**
     * Remove a node from this graph.
     * 
     * @param label
     *            the label of the node to remove.
     * @return the label of the removed node or null if the node did not exist.
     */
    public E removeNode(E label);

    /**
     * Return the set of nodes of this graph.
     * 
     * @return the set of nodes of this graph.
     */
    public Set<E> getNodes();

    /**
     * Return an iterator for the nodes of this graph.
     * 
     * @return an iterator for the nodes of this graph.
     */
    public Iterator<E> nodesIterator();

    /**
     * Add an edge to this graph.
     * 
     * @param node1
     *            source node (in case of directed graph).
     * @param node2
     *            target node (in case of directed graph).
     * @param label
     *            label of this edge.
     * @return true if the edge has been added, false otherwise.
     */
    public boolean addEdge(E node1, E node2, V label);

    /**
     * Remove an edge betweeen two nodes
     * 
     * @param node1
     *            source node (in case of directed graph).
     * @param node2
     *            target node (in case of directed graph).
     * @return the label of the removed edge or null if the edge did not exist.
     */
    public V removeEdge(E node1, E node2);

    /**
     * Return the edge between two nodes.
     * 
     * @param node1
     *            source node (in case of directed graph).
     * @param node2
     *            target node (in case of directed graph).
     * @return the edge or null if it does not exist.
     */
    public Edge<E, V> getEdge(E node1, E node2);

    /**
     * Tell if an edge is present in this graph.
     * 
     * @param node1
     *            source node (in case of directed graph).
     * @param node2
     *            target node (in case of directed graph).
     * @return true if there is an edge in this graph between the given nodes,
     *         false otherwise.
     */
    public boolean containsEdge(E node1, E node2);

    /**
     * Get the color of a given node.
     * 
     * @param node
     *            the node.
     * @return the color of the given node.
     */
    public int getColor(E node);

    /**
     * Set the color of a given node.
     * 
     * @param node
     *            the node.
     * @param color
     *            the color to set.
     */
    public void setColor(E node, int color);

    /**
     * Restituisce il grado di un nodo.
     * 
     * @param label
     *            il nodo.
     * @return il grado del nodo in questo grafo.
     */
    public int degree(E node);

    /**
     * Restituisce il numero di archi in questo grafo.
     * 
     * @return il numero di archi in questo grafo.
     */
    public int edgeCount();

    /**
     * Restituisce un iteratore su tutti i nodi adiacenti a un certo nodo.
     * 
     * @param node
     *            il nodo.
     * @return un iteratore su tutti i nodi adiacenti a node.
     */
    public Iterator<E> neighbors(E node);

    /**
     * Restituisce un iteratore su tutti gli archi di questo grafo.
     * @return un iteratore su tutti gli archi di questo grafo.
     */
    public Iterator<Edge<E, V>> getEdges();

   /**
    * Dice se questo grafo è diretto oppure no.
    * @return true se questo grafo è diretto, false altrimenti.
    */
    public boolean isDirected();

}
